@@ -47,24 +47,30 @@ namespace bareFTP.Gui.FileManager
 		public event TransferRequestHandler TransferRequest;
 		public event ChmodRequestHandler ChmodRequested;
 		public event EventHandler NewDirRequested;
+		public event EventHandler SendCommandRequested;
 		
-		private bool IsRemote;
 		private bareFTP.Preferences.Config conf;
 		private bool eventsDisabled = false;
+		private List<TreePath> dragItems;
+		private bareFTP.Protocol.ProtocolType ptype;
 		
-		public FileList(TreeModel model, bool isremote, bareFTP.Preferences.Config config) : base(model)
+		public FileList(TreeModel model, bareFTP.Protocol.ProtocolType ptype, bareFTP.Preferences.Config config) : base(model)
 		{
-			IsRemote = isremote;
+			this.ptype = ptype;
 			this.conf = config;
 			
 			this.RowActivated += new Gtk.RowActivatedHandler (OnRowActivated);
 			this.KeyPressEvent += new Gtk.KeyPressEventHandler(OnKeyPressEvent);
 			this.Selection.Mode = Gtk.SelectionMode.Multiple;
 			
+			DragBegin += HandleHandleDragBegin;
+			DragMotion += HandleHandleDragMotion;
 			//this.EnableGridLines = Gtk.TreeViewGridLines.Vertical;
 			//this.RubberBanding = true;
 			//RulesHint = true;
 			
+			dragItems = new List<TreePath>();
+			
 			Gtk.TreeViewColumn fileColumn = new Gtk.TreeViewColumn ();
 			fileColumn.Title = Catalog.GetString("Filename");
 			fileColumn.SortColumnId = 0;
@@ -138,6 +144,63 @@ namespace bareFTP.Gui.FileManager
 				col.Sizing = TreeViewColumnSizing.Fixed;
 			}
 		}
+
+		[GLib.ConnectBefore] 
+		void HandleHandleDragMotion (object o, DragMotionArgs args)
+		{
+			
+			/*
+			TreePath path;
+			TreeIter iter;
+       		TreeViewDropPosition pos;
+       		TreeViewDropPosition overrideDrop;
+			
+			if (!GetDestRowAtPos (args.X, args.Y, out path, out pos))
+               return;
+			
+			overrideDrop = pos;
+			Model.GetIter(out iter, path);
+			FileTreeNode node = (FileTreeNode)Model.GetValue(iter,0);
+			
+			if(pos == TreeViewDropPosition.IntoOrAfter)
+			{
+				if(!node.IsDir)
+					overrideDrop = TreeViewDropPosition.After;
+			}
+			else if(pos == TreeViewDropPosition.IntoOrBefore)
+			{
+				if(!node.IsDir)
+					overrideDrop = TreeViewDropPosition.Before;
+			}
+			else
+				return;
+			
+        	SetDragDestRow (path, overrideDrop);
+        	Gdk.Drag.Status (args.Context, args.Context.SuggestedAction, args.Time);
+        	args.RetVal = true; 
+        	*/
+		}
+		
+		public bareFTP.Protocol.ProtocolType ProtocolType
+		{
+			set { ptype = value; }
+		}
+		
+		void HandleHandleDragBegin (object o, DragBeginArgs args)
+		{
+			if(eventsDisabled)
+				return;
+			
+			if(SelectedFileNodes.Count > 0)
+			{
+				dragItems.Clear();
+				
+				foreach(Gtk.TreePath path in Selection.GetSelectedRows())
+					dragItems.Add(path);
+				
+				Gtk.Drag.SetIconPixbuf (args.Context, SelectedFileNodes[0].Type, -10, -10);
+			}
+		}
 		
 		public string ColumnWidths
 		{
@@ -163,7 +226,7 @@ namespace bareFTP.Gui.FileManager
 						}
 					}
 				}
-				catch(Exception ex)
+				catch
 				{
 					Console.WriteLine("Couldn't parse column widths");	
 				}
@@ -191,14 +254,6 @@ namespace bareFTP.Gui.FileManager
 			Model = model;
 		}
 		
-		protected override void OnDragBegin (Gdk.DragContext context)
-		{
-			if(eventsDisabled)
-				return;
-			
-			Gtk.Drag.SetIconDefault(context);
-		}
-
 		private void RenderFileName (Gtk.TreeViewColumn column, Gtk.CellRenderer cell, Gtk.TreeModel model, Gtk.TreeIter iter)
 		{
 			    
@@ -231,7 +286,7 @@ namespace bareFTP.Gui.FileManager
 		{
 			FileTreeNode node = (FileTreeNode)model.GetValue(iter, 0);
 			if(node.Date < DateTime.MaxValue)
-				(cell as MyCellRendererText).Text = node.Date.ToString();
+				(cell as MyCellRendererText).Text = node.Date.ToString("r");
 			else
 				(cell as MyCellRendererText).Text = string.Empty;
 		}
@@ -276,6 +331,11 @@ namespace bareFTP.Gui.FileManager
     		}
     	}
 		
+		public List<TreePath> DragItems
+    	{
+    		get { return dragItems; }
+    	}
+		
 		public List<FileTreeNode> SelectedFileNodes
 		{
 			get
@@ -293,6 +353,13 @@ namespace bareFTP.Gui.FileManager
 				return files;
 			}
 		}
+		
+		public void SetSelection(List<TreePath> items)
+		{
+			Selection.UnselectAll();
+			foreach(TreePath p in items)
+				Selection.SelectPath(p);
+		}
 				
 		#endregion
 				
@@ -422,7 +489,8 @@ namespace bareFTP.Gui.FileManager
 				return null;
 			}
 		}
-
+		
+		/*
 		protected override bool OnButtonPressEvent (Gdk.EventButton evnt)
 		{
 			
@@ -433,9 +501,11 @@ namespace bareFTP.Gui.FileManager
 			}
 			return base.OnButtonPressEvent(evnt);
 		}
-		
-    	private void ShowPopup ()
+		*/
+    	public void ShowPopup ()
     	{
+			if(eventsDisabled)
+				return;
 			List<FileTreeNode> files = new List<FileTreeNode>();
 			Gtk.TreeIter iter;
 			Gtk.TreePath[] paths = this.Selection.GetSelectedRows();
@@ -445,7 +515,7 @@ namespace bareFTP.Gui.FileManager
 				files.Add((FileTreeNode)this.Model.GetValue(iter,0));
 			}
 
-			using(ContextMenu menu = new ContextMenu(IsRemote, files.Count))
+			using(ContextMenu menu = new ContextMenu(ptype, files.Count))
 			{
      			FileListActionArgs _args = new FileListActionArgs(files);
      				
@@ -480,6 +550,16 @@ namespace bareFTP.Gui.FileManager
 				};
 				menu.RefreshDirRequested += delegate {
 					DirectoryChanged(new DirectoryChangedArgs(string.Empty));
+				};
+				menu.SendCommandRequested += delegate {
+					bareFTP.Gui.Dialog.SendCommand scd = new bareFTP.Gui.Dialog.SendCommand();
+					if(scd.Run() == (int)ResponseType.Ok)
+					{
+						SendCommandRequested.Invoke(scd.Command, null);
+					}
+					scd.Destroy();
+					
+					
 				};
 				menu.Popup();
 				menu.ShowAll();
@@ -589,6 +669,21 @@ namespace bareFTP.Gui.FileManager
 			set { eventsDisabled = value; }			
 		}
 		
+		public bool IsClickedNodeSelected (int x, int y)
+		{
+			Gtk.TreePath path;
+			if (GetPathAtPos (x, y, out path))
+				return Selection.PathIsSelected (path);
+			else
+				return false;
+		}
+		
+		public bool MultipleNodesSelected ()
+		{
+			return Selection.GetSelectedRows ().Length > 1;
+		}
+
+		
 #endregion
 		
 	}
