@@ -100,7 +100,7 @@ namespace bareFTP.Gui.FileManager
 			this.PackStart(hbox, false, false, 0);
 			
 			Gtk.ScrolledWindow scrollw = new Gtk.ScrolledWindow();
-			filelist = new FileList(new Gtk.ListStore (typeof (FileTreeNode)),true, config);
+			filelist = new FileList(new Gtk.ListStore (typeof (FileTreeNode)), bareFTP.Protocol.ProtocolType.UNDEFINED, config);
 			
 			Gtk.TargetEntry[] te1 = new TargetEntry[] { 
 				new TargetEntry("treeViewLocal", Gtk.TargetFlags.App, 0),
@@ -110,10 +110,17 @@ namespace bareFTP.Gui.FileManager
 				new TargetEntry("text/uri-list", Gtk.TargetFlags.OtherApp, 1),
 				new TargetEntry("treeViewRemote", Gtk.TargetFlags.App, 0)};
 			
-			filelist.EnableModelDragSource(Gdk.ModifierType.Button1Mask, te2, Gdk.DragAction.Move | Gdk.DragAction.Move);
-			filelist.EnableModelDragDest(te1, Gdk.DragAction.Copy);
+			//filelist.EnableModelDragSource(Gdk.ModifierType.Button1Mask, te2, Gdk.DragAction.Move | Gdk.DragAction.Move);
+			Gtk.Drag.SourceSet (filelist, Gdk.ModifierType.Button1Mask, te2, Gdk.DragAction.Copy);
+			Gtk.Drag.DestSet(filelist, DestDefaults.All, te1, Gdk.DragAction.Copy);
+			//filelist.EnableModelDragDest(te1, Gdk.DragAction.Copy);
+			
 			filelist.DragDataGet += OnDragDataGet;
 			filelist.DragDataReceived += OnDragDataReceived;
+			
+			filelist.ButtonPressEvent += OnButtonPress;
+			filelist.ButtonReleaseEvent += OnButtonRelease;
+			
 			filelist.DirectoryChanged += new DirectoryChangedHandler(OnDirChanged);
 			filelist.NewDirectoryNamed += DirectoryNamed;
 			filelist.FileRenameRequest += new FileRenameRequestHandler(OnFileRenameRequest);
@@ -121,6 +128,7 @@ namespace bareFTP.Gui.FileManager
 			filelist.TransferRequest += new TransferRequestHandler(OnTransferRequested);
 			filelist.ChmodRequested += new ChmodRequestHandler(OnChmodRequested);
 			filelist.LinkActivated += new LinkActivatedHandler(OnLinkActivated);
+			filelist.SendCommandRequested += HandleFilelistSendCommandRequested;
 			filelist.NewDirRequested += OnNewDirClicked;
 			filelist.CdUp += CdUp;;
 			filelist.ColumnWidths = config.GUI_RightColWidths;
@@ -132,6 +140,11 @@ namespace bareFTP.Gui.FileManager
 			this.PackStart(scrollw);
 			
 		}
+
+		void HandleFilelistSendCommandRequested (object sender, EventArgs e)
+		{
+			conn.SendCommand(sender.ToString());
+		}
 		
 		public string ColumnWidths
 		{
@@ -238,6 +251,7 @@ namespace bareFTP.Gui.FileManager
 		{
 			pathEntry.Sensitive = true;
 			this.conn = conn;
+			filelist.ProtocolType = conn.Protocol;
 			IsConnected = true;
 			cache = new FileListCache(180);
 			ChangeDir(startDir, false);
@@ -344,7 +358,7 @@ namespace bareFTP.Gui.FileManager
 
 		private void Traverser(bareFTP.Protocol.BarePath path, List<RemoteFile> _paths)
 		{
-			
+							
 			conn.SetCurrentDirectory(path.RemotePathAbs);
 			
 			foreach(RemoteFile file in conn.XDir())
@@ -371,19 +385,45 @@ namespace bareFTP.Gui.FileManager
 					
 				}
 			}
+			
+		}
+		
+		[GLib.ConnectBefore]
+		private void OnButtonPress (object sender, Gtk.ButtonPressEventArgs args)
+		{
+			bool withModifider = (args.Event.State & Gdk.ModifierType.ShiftMask) != 0 || (args.Event.State & Gdk.ModifierType.ControlMask) != 0;
+			if (filelist.IsClickedNodeSelected ((int)args.Event.X, (int)args.Event.Y) && filelist.MultipleNodesSelected () && !withModifider) {
+				args.RetVal = true;
+			}
+			
+		}
+
+		private void OnButtonRelease (object sender, Gtk.ButtonReleaseEventArgs args)
+		{
+			if (args.Event.Button == 3) {
+				filelist.ShowPopup ();
+			}
+			
+			bool withModifider = (args.Event.State & Gdk.ModifierType.ShiftMask) != 0 || (args.Event.State & Gdk.ModifierType.ControlMask) != 0;
+			if (args.Event.Button == 1 && filelist.MultipleNodesSelected () && !withModifider) {
+				filelist.Selection.UnselectAll ();
+				Gtk.TreePath path;
+				if (filelist.GetPathAtPos ((int)args.Event.X, (int)args.Event.Y, out path))
+					filelist.Selection.SelectPath (path);
+			}
 		}
 		
 		private void OnDragDataGet(object o, DragDataGetArgs args)
 		{
 			string str = string.Empty;
-			if(filelist != null && filelist.SelectedFileNodes.Count > 0)
+			filelist.SetSelection(filelist.DragItems);
+			if(filelist != null && filelist.DragItems.Count > 0)
 			{
 				Gnome.Vfs.Vfs.Initialize();
 				
-				foreach(FileTreeNode node in filelist.SelectedFileNodes)
+				foreach(FileTreeNode item in filelist.SelectedFileNodes)
 				{
-					
-					Gnome.Vfs.Uri uri = new Gnome.Vfs.Uri(conn.Protocol.ToString().ToLower() + "://" + cwd.ToString(true) + "/" +node.FileName);
+					Gnome.Vfs.Uri uri = new Gnome.Vfs.Uri(conn.Protocol.ToString().ToLower() + "://" + cwd.ToString(true) + "/" + item.FileName);
 					uri.HostName = conn.ConnProperties.Hostname;
 					uri.UserName = conn.ConnProperties.User;
 					str += uri.ToString() +  "\n";
@@ -401,6 +441,7 @@ namespace bareFTP.Gui.FileManager
 			Gtk.TreeIter iter;
 			Gtk.TreeViewDropPosition pos;
 			filelist.GetDestRowAtPos(args.X, args.Y, out path, out pos);
+			
 			if(path != null)
 			{
 				filelist.Model.GetIter(out iter, path);
@@ -442,31 +483,35 @@ namespace bareFTP.Gui.FileManager
 		
 		private void DoTraverse(object obj)
 		{
-			List<RemoteFile> paths = obj as List<RemoteFile>;
-			foreach(FileTreeNode fnode in filelist.SelectedFileNodes)
+			try
 			{
-				if(fnode.FileName == "..")
-					continue;
+				List<RemoteFile> paths = obj as List<RemoteFile>;
+				foreach(FileTreeNode fnode in filelist.SelectedFileNodes)
+				{
+					if(fnode.FileName == "..")
+						continue;
+						
+					RemoteFile rfile = new RemoteFile();
+					rfile.Path.SetBaseRemote(cwd.ToString(true));
 					
-				RemoteFile rfile = new RemoteFile();
-				rfile.Path.SetBaseRemote(cwd.ToString(true));
-				
-				rfile.Size = Convert.ToInt64(fnode.Size);
-				rfile.IsDir =  fnode.IsDir;
-				rfile.IsLink = fnode.IsLink;
-				rfile.Linkdest = fnode.LinkName;
-				if(!string.IsNullOrEmpty(fnode.Rights))
-					rfile.Permissions = PermissionParser.RWXToMode(fnode.Rights);
-				
-				if(!rfile.IsDir)
-					rfile.Path.SetFileName(string.Empty, fnode.FileName);
-				else
-					rfile.Path.SetFileName(fnode.FileName, string.Empty);
-				
-				paths.Add(rfile);
-				if(rfile.IsDir)
-					Traverser(rfile.Path, paths);
+					rfile.Size = Convert.ToInt64(fnode.Size);
+					rfile.IsDir =  fnode.IsDir;
+					rfile.IsLink = fnode.IsLink;
+					rfile.Linkdest = fnode.LinkName;
+					if(!string.IsNullOrEmpty(fnode.Rights))
+						rfile.Permissions = PermissionParser.RWXToMode(fnode.Rights);
+					
+					if(!rfile.IsDir)
+						rfile.Path.SetFileName(string.Empty, fnode.FileName);
+					else
+						rfile.Path.SetFileName(fnode.FileName, string.Empty);
+					
+					paths.Add(rfile);
+					if(rfile.IsDir)
+						Traverser(rfile.Path, paths);
+				}
 			}
+			catch{}
 			
 			ewh.Set();
 		}
