@@ -80,8 +80,7 @@ namespace bareFTP.Protocol.Ftp
 		
 		public void Connect(string remoteHost, int remotePort, string user, string password)
 		{
-			
-			tcpClient.Connect(bareFTP.Protocol.HostNameResolver.GetAddress(remoteHost), remotePort);
+			tcpClient.Connect(bareFTP.Protocol.HostResolver.GetAddress(remoteHost), remotePort);
 			Stream stream = tcpClient.GetStream();
 			
             writer = new StreamWriter(stream, System.Text.Encoding.ASCII);
@@ -116,10 +115,9 @@ namespace bareFTP.Protocol.Ftp
 				
 				try
 				{
-					//sslstream.AuthenticateAsClient(remoteHost, 
 					sslstream.AuthenticateAsClient(remoteHost);
 				}
-				catch
+				catch(Exception ex)
 				{
 					
 					reader.Close();
@@ -133,9 +131,11 @@ namespace bareFTP.Protocol.Ftp
 					sslstream.Dispose();
 					sslstream = null;
 					
-					throw new Exception("Certificate Error");
+					//Console.WriteLine(ex.ToString());
+					
+					throw new Exception(string.Format("Certificate Error ({0})", ex.Message));
 				}
-		
+				
 				writer = new StreamWriter(sslstream, GetEncoding());
 				reader = new StreamReader(sslstream, GetEncoding());
 				
@@ -144,12 +144,12 @@ namespace bareFTP.Protocol.Ftp
 			}
 			
 			try
-			{
+			{ 
 				CheckReply(SendCommand("USER " + user), 331);
 				CheckReply(SendCommand("PASS " + password), 230);
 			}
 			catch(Exception e)
-			{
+			{ 
 				stream.Close();
 				stream.Dispose();
 				stream = null;
@@ -225,7 +225,29 @@ namespace bareFTP.Protocol.Ftp
 			writer.BaseStream.Write(cmdBytes, 0, cmdBytes.Length);
 			writer.Flush();
 			return GetReply();
-			
+		}
+		
+		public void SendCustomCommand(string command)
+		{
+			List<string> lines = new List<string>();
+			DataSocket dsocket = CreateDataSocket();
+			SetTransferType(FTPFileTransferType.ASCII);
+			List<FTPReply> replies = SendCommand(command);
+			CheckReply(replies, -1);
+			foreach(FTPReply r in replies)
+			{
+				if(r.ReplyCode == 125	 || r.ReplyCode == 150)
+				{
+					lines = ReadLines(dsocket.GetStream());
+					dsocket.Close();
+					foreach(string s in lines)
+					OnLogTextEmitted(new LogTextEmittedArgs(MessageType.Info, s));
+					// Read reply from control socket. Should be 226
+					CheckReply(GetReply(), -1);
+					return;
+				}
+			}
+			dsocket.Close();
 		}
 		
 		public void ChangeDir(string dir)
@@ -437,6 +459,7 @@ namespace bareFTP.Protocol.Ftp
 		
 		public List<FTPReply> GetReply()
 		{
+			
 			try
 			{
 				List<FTPReply> replies = new List<FTPReply>();
@@ -464,7 +487,7 @@ namespace bareFTP.Protocol.Ftp
 			
 				if (line.Length > 4)
 					reply = line.Substring(4);
-			
+				
 				// check for multiline response and build up
 				// the reply
 				if (line[3] == '-')
@@ -541,12 +564,17 @@ namespace bareFTP.Protocol.Ftp
 		
 		internal List<string> ReadLines(Stream stream)
 		{
+			
 			List<string> lines = new List<string>();
 			
+			if(stream == null)
+				throw new Exception("Input stream is null");
+			
 			using(StreamReader input = new StreamReader(stream, GetEncoding()))
 			{
 				
 				string line = null;
+				
 				while ((line = ReadLine(input)) != null)
 					lines.Add(line);
             
@@ -558,8 +586,8 @@ namespace bareFTP.Protocol.Ftp
 		internal virtual string ReadLine(TextReader input)
         {
 			return input.ReadLine();
-        }
-		
+		}
+			
 		internal DataSocket CreateDataSocket()
         {            
 			
@@ -696,7 +724,7 @@ namespace bareFTP.Protocol.Ftp
 			foreach(FTPReply r in reply)
 			{
 				msgstr += r.Message + System.Environment.NewLine;
-				if(codelist.Contains(r.ReplyCode))
+				if(codelist.Contains(r.ReplyCode) || codelist.Contains(-1))
 					return reply;
 			}
 			
@@ -732,6 +760,5 @@ namespace bareFTP.Protocol.Ftp
 			LogTextEmitted(this, e);
 		}
 		
-		
 	}
 }
