@@ -143,7 +143,7 @@ namespace bareFTP.Protocol.Sftp
 			
 			pty = bareFTP.SftpPty.PseudoTerminal.Open (null,
 					       conf.SSHProgramPath,
-					       new string [] {conf.SSHProgramPath, hostname, "-l", user, "-e", "none", "-s", "sftp" },
+					       new string [] {conf.SSHProgramPath, hostname, "-l", user, "-p", port.ToString(), "-e", "none", "-s", "sftp" },
 					       null);
 			
 			consoleReader = new UnixStream (pty.FileDescriptor, false);
@@ -246,16 +246,19 @@ namespace bareFTP.Protocol.Sftp
 			if(isConnected)
 			{
 				
-				//TODO: Overkill, but still leaves a defunct ssh process..
-				/*
-				Mono.Unix.Native.Syscall.kill(pty.ChildPid, Mono.Unix.Native.Signum.SIGHUP);
-				Mono.Unix.Native.Syscall.kill(pty.ChildPid, Mono.Unix.Native.Signum.SIGQUIT);
-				*/
+				int _signal = 0;
+				
 				Mono.Unix.Native.Syscall.close(pty.SlaveFileDescriptor);
 				Mono.Unix.Native.Syscall.close(pty.FileDescriptor);
+				
+				Mono.Unix.Native.Syscall.kill(pty.ChildPid, Mono.Unix.Native.Signum.SIGQUIT);
+				Mono.Unix.Native.Syscall.waitpid(pty.ChildPid, out _signal, Mono.Unix.Native.WaitOptions.WUNTRACED);
+				
 				dataStream.Close();
 				consoleReader.Close();
 				consoleWriter.Close();
+				
+				pty.Dispose();
 				pty = null;
 				isConnected = false;
 			}
