@@ -57,7 +57,7 @@ namespace bareFTP
 		private AccelGroup agroup;
 		
 		
-		public MainWindow (Preferences.Config conf): base (Gtk.WindowType.Toplevel)
+		public MainWindow (Preferences.Config conf, bool debug): base (Gtk.WindowType.Toplevel)
 		{
 			
 			Mono.Unix.Catalog.Init("bareftp", Defines.LOCALE_DIR);
@@ -309,21 +309,6 @@ namespace bareFTP
 				}
 			}
 			
-			/*
-			if(conf.General_SimultaneousTransfers)
-				xferfiles.Sort( delegate(XferFile file0, XferFile file1)
-							{
-					if(file0.IsDir == file1.IsDir)
-						return 0;
-					if(file0.IsDir && !file1.IsDir)
-						return -1;
-					if(!file0.IsDir && file1.IsDir)
-						return 1;
-					else
-						return 0;
-				});
-			*/
-			
 			if(showFileActionDialog)
 			{
 				AskFileAction md = new AskFileAction(xferfiles);
@@ -686,11 +671,11 @@ namespace bareFTP
 			conf.GUI_LeftColWidths = file_local.ColumnWidths;
 			try
 			{
-			if(pq != null)
-				pq.AbortAllTransfers();
-			if(conn != null)
-				if(conn.Connected)
-					conn.Close();
+				if(pq != null)
+					pq.AbortAllTransfers();
+				if(conn != null)
+					if(conn.Connected)
+						conn.Close();
 			}
 			catch(Exception ex)
 			{
@@ -700,6 +685,7 @@ namespace bareFTP
 		protected virtual void onMenuQuit (object sender, System.EventArgs e)
 		{
 			CleanUpBeforeExit();
+			
 			Application.Quit ();
 		}
 		
@@ -933,8 +919,8 @@ namespace bareFTP
 				{
 					Gtk.Application.Invoke( 
 			  			delegate {
-							SetStatusBarMessage("ERROR: " + ex.Message, true);
-							
+							//SetStatusBarMessage("ERROR: " + ex.Message, true);
+						OnLogTextEmitted(this, new LogTextEmittedArgs(bareFTP.Protocol.MessageType.Error, ex.Message));
 						});
 				}
 				
@@ -1025,27 +1011,44 @@ namespace bareFTP
 				Gtk.Frame f = null;
 				Gtk.Label l = null;
 				
-				foreach(Widget w in statusbar1.Children)
+				IEnumerator ie = statusbar1.AllChildren.GetEnumerator();
+				while(ie.MoveNext())
 				{
-					if(w is Gtk.Frame)
+					if(ie.Current is Frame)
 					{
-						f = w as Gtk.Frame;
-						foreach(Widget w2 in f.Children)
+						f = (Frame)ie.Current;
+						IEnumerator i2 = f.AllChildren.GetEnumerator();
+						while(i2.MoveNext())
 						{
-							if(w2 is Label)
-								l = w2 as Label;
+							if(i2.Current is Label)
+							{
+								l = (Label)i2.Current;
+								break;
+							}
+							if(i2.Current is HBox)
+							{
+								HBox b = (HBox)i2.Current;
+								IEnumerator i3 = b.AllChildren.GetEnumerator();
+								while(i3.MoveNext())
+								{
+									if(i3.Current is Label)
+									l = (Label)i3.Current;
+									break;
+								}
+							}
+							
 						}
 					}
 				}
 				
-				if(l == null)
-					return;
-				
+				if(l != null)
+				{
 					Gdk.Color scol = Gdk.Color.Zero;
 					if(!Gdk.Color.Parse(color, ref scol))
 						throw new Exception("No such color");
 					l.ModifyFg(StateType.Normal, scol);
 					l.UseMarkup = true;
+				}
 			}
 			catch{}
 			
